/*
*
*  RMLoader.java
*
*  SPECT: Simple Portal-Enhancing Configuration Tool
*  Copyright (C) 1999, 2000 Ray Ontko & Co.
*  http://www.ontko.com/
*
*  This program is a simple-to-use configuration tool for Brio.Portal
*  (formerly ReportMart).
*
*  "Brio.Portal" is a trademark of Brio Technology, http://www.brio.com/ .
* 
* 
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
*    http://www.gnu.org/copyleft/gpl.html
* 
*/ 

import java.io.*;
import java.util.*;
import com.sqribe.rm.*;

/*****************************************************************
*
* RMLoader is the entrance point for the program.  This class manages
* files, output, and all action objects.
*
******************************************************************/
public class RMLoader {

  // Constants that indicate the type of action specified
  public static final int SETCONFIG     = 0;
  public static final int SET           = 1;
  public static final int CREATE        = 2;
  public static final int UPDATE        = 3;
  public static final int CREATE_UPDATE = 4;
  public static final int DELETE        = 5;
  public static final int ADD           = 6;
  public static final int REMOVE        = 7;

  // Static variables
  public static String infile;
  public static String username;
  public static String group;
  public static Vector action_list;
  public static RMConfig config;
  public static PrintWriter good, bad, log;
  public static PrintWriter err;
  private static Vector file_list;

  // Session and action variables
  private Session session;
  private Repository repository;
  private StreamTokenizer st;
  private UserAction ua;
  private GroupAction ga;
  private UserGroupsAction uga;
  private GroupUsersAction gua;
  private CategoryAction ca;
  private FileAction fa;

  static {
    file_list = new Vector();
    action_list = new Vector();
    action_list.addElement("SETCONFIG");
    action_list.addElement("SET");
    action_list.addElement("CREATE");
    action_list.addElement("UPDATE");
    action_list.addElement("CREATE-UPDATE");
    action_list.addElement("DELETE");
    action_list.addElement("ADD");
    action_list.addElement("REMOVE");

    log = new PrintWriter(System.out, true);
    err = new PrintWriter(System.out, true);
    good = null;
    bad = null;
  }

  /**
  *
  * The constructor creates the ReportMart session using the SessionFactory
  * class, opens the control file as a StreamTokenizer, and calls the parse()
  * method.
  *
  **/
  public RMLoader(String name, String pwd, String host, Integer port,
                  String infile) {
    try {
      this.infile = infile;
      username = name;
      if (port == null)
         session = SessionFactory.getInstance(name,pwd,host);
      else
         session = SessionFactory.getInstance(name,pwd,host,port.intValue());
      group = session.getRepository().getUser(username).getGroup().getName();
      config = new RMConfig();

      // Set the options for the stream tokenizer.  The special 
      // characters are:
      //  '#' - comment character
      //  '"' - quote character
      //  '=' - used to link value to a keyword
      //  ';' - indicates end of command
      //  <space>, <tab> - white-space characters--used to delimit
      //                   tokens (when not contained in quotes)
      //  <EOL> - end-of-line is ignored
      // All other characters are considered to be part of a token and
      // are not treated individually.  Note: this initialization
      // assumes an ASCII or compatible character set.
      st = new StreamTokenizer(new FileReader(infile));
      st.slashSlashComments(true);
      st.slashStarComments(true);
      st.eolIsSignificant(false);
      st.commentChar('#');
      st.quoteChar('"');
      st.ordinaryChar('=');
      st.ordinaryChar(';');
      st.wordChars('!','!');
      st.wordChars('$',':');
      st.wordChars('<','<');
      st.wordChars('>','~');
      repository = session.getRepository();
      parse();
      session.close();
    }
    catch(ReportMartException rme) {
      System.out.println("Error creating session: " + rme.getMessage());
    }
    catch(FileNotFoundException fnfe) {
      System.out.println("File not found: \"" + infile + "\"");
    }
    catch(IOException ioe) {
      System.out.println("Error reading file: \"" + infile + "\"");
    }
  }

  /**
  *
  * parse() reads the action and object-type keywords from the control
  * file and creates the appropriate action object.
  *
  **/
  private void parse() throws IOException {
    String action_text;
    String object_type;
    int action;
    int token_type;
    
    while (true) {
      st.nextToken();
      if (st.ttype == st.TT_EOF)
         break;
      int lineno = st.lineno();

      // Look for an action keyword.
      if (st.ttype == st.TT_WORD || st.ttype == '"') {
        action_text = st.sval;
        action = encodeAction(action_text);
        if (action < 0) {
          error(st.lineno(), "invalid action: " + action_text);
          recover(st);
        }
        else 
          if (action == SETCONFIG) {
             config.parse(st);
          }
          else if (st.nextToken() == st.TT_WORD || st.ttype == '"') {
            object_type = st.sval;
            if (object_type.equalsIgnoreCase("USER")) {
              ua = new UserAction(repository, action, st, lineno);
            }
            else if (object_type.equalsIgnoreCase("GROUP")) {
              ga = new GroupAction(repository, action, st, lineno);
            }
            else if (object_type.equalsIgnoreCase("CATEGORY")) {
              ca = new CategoryAction(repository, action, st, lineno);
            }
            else if (object_type.equalsIgnoreCase("USERGROUPS")) {
              uga = new UserGroupsAction(repository, action, st, lineno);
            }
            else if (object_type.equalsIgnoreCase("GROUPUSERS")) {
              gua = new GroupUsersAction(repository, action, st, lineno);
            }
            else if (object_type.equalsIgnoreCase("FILE")) {
              fa = new FileAction(session, repository, action, st, lineno);
            }
            else if (object_type.equalsIgnoreCase("JOB")) { 
              System.out.println("(JOB object not implemented yet)");
            }
            else {
              error(st.lineno(), "Invalid object type: \"" + 
                    object_type + "\"");
            }
          }  /* if (st.ttype == st.TT_WORD || st.ttype == '"') */
          else {
            error(st.lineno(), "Invalid character found instead of object " +
                  "type name: \"" + new Character((char)st.ttype) + "\"");
          }
      } /* if (st.ttype == st.TT_WORD || st.ttype == '"') */
      else {
        error(st.lineno(), "Invalid character found instead of action " +
             ": \"" + new Character((char)st.ttype) + "\"");
      }
    // make sure we're at the end of the statement.
    // (If the statement was valid, we'll already be there, but if any 
    // errors were found, this we'll skip to the end of it here.) 
    recover(st);
    } /* while (true) */
  } /* private void parse() */

  /**
  *
  * If a syntactical error is encountered in the control file, recover()
  * reads tokens until the end-of-statement character (';') is read.
  *
  **/
  public static void recover(StreamTokenizer st) throws IOException{
    while ( ! (st.ttype == st.TT_EOF || st.ttype == ';') ) {
      st.nextToken();
    }
  }
        
  public static void error(int lineno, String msg) {
    err.println("Error line " + lineno + ": " + msg);
  }

  public static void warning(int lineno, String msg) {
    err.println("Warning line " + lineno + ": " + msg);
  }

  /**
  *
  * When a SETCONFIG good-file=xxxx is encountered in the control file,
  * this method is called (from RMConfig).  Opens filename as good.
  *
  **/
  public static void openGoodFile(String filename){
    try {
      if (filename.startsWith(">>"))
        good = new PrintWriter(new FileWriter(filename.substring(2),true),true);
      else if (filename.equals("*"))
        good = null;
      else if (filename.equals("-"))
        good = new PrintWriter(System.out,true);
      else
        good = new PrintWriter(new FileWriter(filename, 
          shouldAppend(filename)), true);
    }
    catch (IOException ioe) {
      System.out.println("Error opening good file \"" + filename + "\"");
    }
  }

  /**
  *
  * When a SETCONFIG bad-file=xxxx is encountered in the control file,
  * this method is called (from RMConfig).  Opens filename as bad.
  *
  **/
  public static void openBadFile(String filename){
    try {
      if (filename.startsWith(">>"))
        bad = new PrintWriter(new FileWriter(filename.substring(2),true),true);
      else if (filename.equals("*"))
        bad = null;
      else if (filename.equals("-"))
        bad = new PrintWriter(System.out,true);
      else
        bad = new PrintWriter(new FileWriter(filename, 
          shouldAppend(filename)), true);
    }
    catch (IOException ioe) {
      System.out.println("Error opening bad file \"" + filename + "\"");
    }
  }

  /**
  *
  * When a SETCONFIG log-file=xxxx is encountered in the control file,
  * this method is called (from RMConfig).  Opens filename as log.
  *
  **/
  public static void openLogFile(String filename){
    try {
      if (filename.startsWith(">>"))
        log = new PrintWriter(new FileWriter(filename.substring(2),true),true);
      else if (filename.equals("*"))
        log = null;
      else if (filename.equals("-"))
        log = new PrintWriter(System.out,true);
      else
        log = new PrintWriter(new FileWriter(filename, 
          shouldAppend(filename)), true);
      log.println("Output of " + infile);
    }
    catch (IOException ioe) {
      System.out.println("Error opening log file \"" + filename + "\"");
    }
  }

  /**
  *
  * logMsg() prints msg to the log PrintWriter, if it is not null.
  *
  **/
  public static void logMsg(String msg) {
    if (log != null)
      log.println(msg);
  }

  public static int encodeAction(String action) {
    if (action.equalsIgnoreCase("SETCONFIG"))
      return SETCONFIG;
    else if (action.equalsIgnoreCase("SET"))
      return SET;
    else if (action.equalsIgnoreCase("CREATE"))
      return CREATE;
    else if (action.equalsIgnoreCase("UPDATE"))
      return UPDATE;
    else if (action.equalsIgnoreCase("CREATE_UPDATE"))
      return CREATE_UPDATE;
    else if (action.equalsIgnoreCase("DELETE"))
      return DELETE;
    else if (action.equalsIgnoreCase("ADD"))
      return ADD;
    else if (action.equalsIgnoreCase("REMOVE"))
      return REMOVE;
    else return -1;
  }

  public static String decodeAction(int action) {
    switch (action) {
    case SETCONFIG :
      return "SETCONFIG";
    case SET :
      return "SET";
    case CREATE :
      return "CREATE";
    case UPDATE :
      return "UPDATE";
    case CREATE_UPDATE :
      return "CREATE_UPDATE";
    case DELETE :
      return "DELETE";
    case ADD :
      return "ADD";
    case REMOVE :
      return "REMOVE";
    default :
      return null;
    }
  }

  /**
  *
  * RMLoader keeps track of the names of files that have been opened
  * during the program run for logging, printing good/bad records.  When
  * a log/good/bad file is to be opened, shouldAppend() returns true
  * if the file has been previously opened during the program run, and thus 
  * the file should be opened in append mode
  *
  **/
  private static boolean shouldAppend(String filename) {
    if (file_list.contains(filename)) {
      return true;
    }
    else {
      file_list.addElement(filename);
      return false;
    }
  }
}
