/*
*
*  UserAction.java
*
*  SPECT: Simple Portal-Enhancing Configuration Tool
*  Copyright (C) 1999, 2000 Ray Ontko & Co.
*  http://www.ontko.com/
*
*  This program is a simple-to-use configuration tool for Brio.Portal
*  (formerly ReportMart).
*
*  "Brio.Portal" is a trademark of Brio Technology, http://www.brio.com/ .
* 
* 
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
*    http://www.gnu.org/copyleft/gpl.html
* 
*/ 

//#FLAG API_V1.5_ONLY currently ENABLED

import java.io.*;
import java.util.*;
import com.sqribe.rm.*;

/*******************************************************************
*
*  UserAction parses a user-related command from the control file
*  and performs the specified action on ReportMart.  The allowed actions
*  are create user, update user, and delete user.  
*
*******************************************************************/
public class UserAction extends BaseAction {

  private Repository repository;
  private static RMHashtable default_keywords;
  private RMHashtable keywords;
  private StreamTokenizer st;
  private int action;
  private int lineno;

  // Load default values into keyword hashtable
  static {
    default_keywords = new RMHashtable();
    RMProperty gp = new RMProperty("NAME","");
    default_keywords.put(gp.name, gp);
    gp = new RMProperty("DESCRIPTION","");
    default_keywords.put(gp.name, gp);
    gp = new RMProperty("PASSWORD","");
    default_keywords.put(gp.name, gp);
    gp = new RMProperty("GROUP","");
    default_keywords.put(gp.name, gp);
    gp = new RMProperty("HOME-CATEGORY","/");
    default_keywords.put(gp.name, gp);
    gp = new RMProperty("PERMISSIONS","740");
    default_keywords.put(gp.name, gp);
    gp = new RMProperty("SUPERUSER","FALSE");
    default_keywords.put(gp.name, gp);
    gp = new RMProperty("EXECUTE-JOBS","FALSE");
    default_keywords.put(gp.name, gp);
//#IF API_V1.5_ONLY DISABLED
//!    gp = new RMProperty("EMAIL","");
//!    default_keywords.put(gp.name, gp);
//#END API_V1.5_ONLY
  }

  /**
  *
  *  Parses the contents of the command and initiates the action
  *
  **/
  public UserAction(Repository repository, int action, StreamTokenizer st, 
         int lineno) throws IOException{
    super(action, st, lineno);
    this.repository = repository;
    this.action = action;
    this.type = "USER";
    keywords = (RMHashtable)default_keywords.clone();
    parse(default_keywords, keywords);
    doAction();
  }

  /** After successfully parsing a GROUP action, do the action
   ** specified with the keyword values given, or defaulted **/
  public int doAction() {
    switch (action) {
    case RMLoader.CREATE :
      createUser();
      break;
    case RMLoader.UPDATE :
      updateUser();
      break;
    case RMLoader.DELETE :
      deleteUser();
      break;
    default :
      error("invalid action for USER, \"" + RMLoader.decodeAction(action) + 
        "\"");
      return 1;
    }
    return 0;
  }

  /**
  *
  *  Creates a user in ReportMart with the given properties.
  *
  **/
  public void createUser() {
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;

    // check to see if user already exists.  if it doesn't, create
    // it. if it does, check config option WHEN-PREEXISTS to see what
    // to do.
    try {
      repository.getUser(name);
      switch (RMLoader.config.when_preexists) {
      case RMConfig.ERROR :
        error("user \"" + name + "\" already exists, cannot add");
        printBadRecord();
        return;
      case RMConfig.WARNING :
        warning("user \"" + name + "\" already exists; skipping");
        printGoodRecord();
        return;
      case RMConfig.IGNORE :
        printGoodRecord();
        return;
      case RMConfig.UPDATE :
        updateUser();
        return;
      }
    }  
    catch (ReportMartException rme) {} // It doesn't exist.  

    // The user doesn't exist, attempt to add it. 

    // Make sure a password is given
    rmp = (RMProperty)keywords.get("PASSWORD");
    if (rmp.is_default) {
      error("password missing in CREATE USER command, unable to execute");
      printBadRecord();
      return;
    }
    String password = rmp.value;

    // Make sure a group is given
    rmp = (RMProperty)keywords.get("GROUP");
    String groups[] = new String[1];
    groups[0] = rmp.value;
    if (rmp.is_default) {
      error("default group missing in CREATE USER command, unable to execute");
      printBadRecord();
      return;
    }
 
    // Make sure home-category exists
    rmp = (RMProperty)keywords.get("HOME-CATEGORY");
    Category home_category;
    try {
      home_category = repository.getCategory(rmp.value);
    }
    catch (ReportMartException rme) {
      error("unable to find category \"" + rmp.value + "\", " + 
        rme.getMessage());
      printBadRecord();
      return; 
    }

    // Get all other property values
    rmp = (RMProperty)keywords.get("DESCRIPTION");
    String description = rmp.value;
    rmp = (RMProperty)keywords.get("PERMISSIONS");
    Permissions permissions = convertUmask(rmp.value);
    rmp = (RMProperty)keywords.get("SUPERUSER");
    boolean superuser = decodeBoolean(rmp.value);
    rmp = (RMProperty)keywords.get("EXECUTE-JOBS");
    boolean execute_jobs = decodeBoolean(rmp.value);

    User user;
    try {
      user = repository.addUser(name, password, groups, description,
        permissions, home_category, execute_jobs, superuser);
    }
    catch(ReportMartException rme) {
      error("unable to create user \"" + name + "\", " + rme.getMessage());
      printBadRecord();
      return;
    }

//#IF API_V1.5_ONLY DISABLED
//!    rmp = (RMProperty)keywords.get("EMAIL");
//!    if (!rmp.is_default) {
//!      try {
//!        user.setEmailAddress(rmp.value);
//!        user.update();
//!      } 
//!      catch (ReportMartException rme) {
//!        error("unable to set email address for users " + name + ".");
//!        printBadRecord();
//!        return;
//!      }
//!    }
//#END API_V1.5_ONLY
    log("User \"" + name + "\" successfully created");
    printGoodRecord();
  } 

  /**
  *
  * Updates properties of a user previously added to ReportMart
  *
  **/
  private void updateUser() {
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;
    String msg = "";
    User user = null;

    // Check to make sure user exists
    try {
      user = repository.getUser(name);
    }
    catch (ReportMartException rme) {
      switch (RMLoader.config.when_missing) {
      case RMConfig.ERROR :
        error("unable to find user \"" + name + "\", " + rme.getMessage() +
          ", unable to update");
        printBadRecord();
        return;
      case RMConfig.WARNING :
        warning("unable to find user \"" + name + "\", " + rme.getMessage() +
          ", unable to update");
      case RMConfig.IGNORE :
        printGoodRecord();
        return;
      }
    }

    // User exists, now check for ignored keywords
    rmp = (RMProperty)keywords.get("SUPERUSER");
    if (!rmp.is_default) {
      switch(RMLoader.config.when_ignored_keyword) {
      case RMConfig.ERROR :
        error("ignored keyword: SUPERUSER, aborting update");
        printBadRecord();
        return;
      case RMConfig.WARNING :
        warning("ignored keyword: SUPERUSER");
      case RMConfig.IGNORE :
      }
    }

    // now go ahead and update the user
    try {
      rmp = (RMProperty)keywords.get("DESCRIPTION");
      if (!rmp.is_default) {
        msg = "unable to set description, ";
        user.setDescription(rmp.value);
      }
      rmp = (RMProperty)keywords.get("PASSWORD");
      if (!rmp.is_default) {
        msg = "unable to set password, ";
        user.setPassword(rmp.value);
      }
      rmp = (RMProperty)keywords.get("GROUP");
      if (!rmp.is_default) {
        msg = "unable to set group " + rmp.value + " as default group, ";
        Group group = repository.getGroup(rmp.value);
        user.setDefaultGroup(group);
      }
      rmp = (RMProperty)keywords.get("HOME-CATEGORY");
      if (!rmp.is_default) {
        msg = "unable to set category " + rmp.value + " as home category, ";
        Category category = repository.getCategory(rmp.value);
        user.setDefaultCategory(category);
      }
      rmp = (RMProperty)keywords.get("PERMISSIONS");
      if (!rmp.is_default) {
        msg = "unable to set default permissions, ";
        user.setDefaultPermissions(convertUmask(rmp.value));
      }
      rmp = (RMProperty)keywords.get("EXECUTE-JOBS");
      if (!rmp.is_default) {
        msg = "unable to set execute privileges, ";
        user.setExecutePrivilege(decodeBoolean(rmp.value));
      }
//#IF API_V1.5_ONLY DISABLED
//!      rmp = (RMProperty)keywords.get("EMAIL");
//!      if (!rmp.is_default) {
//!        msg = "unable to set email address for users " + name + ", " ;
//!        user.setEmailAddress(rmp.value);
//!      } 
//#END API_V1.5_ONLY
      msg = "";
      user.update();
      log("User \"" + name + "\" successfully updated");
      printGoodRecord();
    }
    catch (ReportMartException rme) {
      error(msg + rme.getMessage() + ", unable to update user \"" + name + "\"");
      printBadRecord();
    }
  }

  /**
  *
  *  Deletes the named user if conditions are met.
  *
  **/
  private void deleteUser() {
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;

    // check for ignored keywords
    Enumeration e = keywords.elements();
    boolean error_status = false;
    while (e.hasMoreElements()) {
      rmp = (RMProperty)e.nextElement();
      if (!rmp.name.equals("NAME") && !rmp.is_default) {
        switch(RMLoader.config.when_ignored_keyword) {
        case RMConfig.ERROR :
          error_status = true;
          error("ignored keyword, " +rmp.name + ", user will not be deleted");
          break;
        case RMConfig.WARNING :
          warning("ignored keyword, " + rmp.name);
        case RMConfig.IGNORE :
        }
      }
    }
    if (error_status) {
      printBadRecord();   
      return;
    }
    try {
      User user = repository.getUser(name);
      repository.removeUser(user);
      log("User \"" + name + "\" successfully deleted");
    }
    catch (ReportMartException rme) {
      // what to do? check config
      error("unable to delete user \"" + name + "\": " + rme.getMessage());
    }
  }
}
